@@ -3,7 +3,7 @@
  dxf_load_distinct.c -- implements DXF support
  [loding features into the DB - by distinct layers]
 
- version 4.1, 2013 May 14
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -25,7 +25,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s): 
@@ -44,6 +44,15 @@ the terms of any one of the MPL, the GPL or the LGPL.
  
 */
 
+/*
+ 
+CREDITS:
+
+inital development of the DXF module has been funded by:
+Regione Toscana - Settore Sistema Informativo Territoriale ed Ambientale
+
+*/
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -1361,6 +1370,7 @@ import_by_layer (sqlite3 * handle, gaiaDxfParserPtr dxf, int append)
 		pg = lyr->first_polyg;
 		while (pg != NULL)
 		  {
+		      int unclosed = check_unclosed_polyg (pg, lyr->is3Dpolyg);
 		      int num_holes;
 		      gaiaDxfHolePtr hole;
 		      sqlite3_reset (stmt);
@@ -1382,7 +1392,8 @@ import_by_layer (sqlite3 * handle, gaiaDxfParserPtr dxf, int append)
 			    num_holes++;
 			    hole = hole->next;
 			}
-		      gaiaAddPolygonToGeomColl (geom, pg->points, num_holes);
+		      gaiaAddPolygonToGeomColl (geom, pg->points + unclosed,
+						num_holes);
 		      p_pg = geom->FirstPolygon;
 		      p_rng = p_pg->Exterior;
 		      for (iv = 0; iv < pg->points; iv++)
@@ -1399,13 +1410,30 @@ import_by_layer (sqlite3 * handle, gaiaDxfParserPtr dxf, int append)
 						*(pg->x + iv), *(pg->y + iv));
 			      }
 			}
+		      if (unclosed)
+			{
+			    /* forcing the Ring to be closed */
+			    if (lyr->is3Dpolyg)
+			      {
+				  gaiaSetPointXYZ (p_rng->Coords, pg->points,
+						   *(pg->x + 0), *(pg->y + 0),
+						   *(pg->z + 0));
+			      }
+			    else
+			      {
+				  gaiaSetPoint (p_rng->Coords, pg->points,
+						*(pg->x + 0), *(pg->y + 0));
+			      }
+			}
 		      num_holes = 0;
 		      hole = pg->first_hole;
 		      while (hole != NULL)
 			{
+			    int unclosed =
+				check_unclosed_hole (hole, lyr->is3Dpolyg);
 			    p_rng =
 				gaiaAddInteriorRing (p_pg, num_holes,
-						     hole->points);
+						     hole->points + unclosed);
 			    for (iv = 0; iv < hole->points; iv++)
 			      {
 				  if (lyr->is3Dpolyg)
@@ -1422,6 +1450,25 @@ import_by_layer (sqlite3 * handle, gaiaDxfParserPtr dxf, int append)
 						      *(hole->y + iv));
 				    }
 			      }
+			    if (unclosed)
+			      {
+				  /* forcing the Ring to be closed */
+				  if (lyr->is3Dpolyg)
+				    {
+					gaiaSetPointXYZ (p_rng->Coords,
+							 hole->points,
+							 *(hole->x + 0),
+							 *(hole->y + 0),
+							 *(hole->z + 0));
+				    }
+				  else
+				    {
+					gaiaSetPoint (p_rng->Coords,
+						      hole->points,
+						      *(hole->x + 0),
+						      *(hole->y + 0));
+				    }
+			      }
 			    num_holes++;
 			    hole = hole->next;
 			}
