@@ -3,7 +3,7 @@
  dxf_writer.c -- implements DXF support
  [exporting a DXF]
 
- version 4.1, 2013 May 27
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -25,7 +25,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s): 
@@ -44,6 +44,15 @@ the terms of any one of the MPL, the GPL or the LGPL.
  
 */
 
+/*
+ 
+CREDITS:
+
+inital development of the DXF module has been funded by:
+Regione Toscana - Settore Sistema Informativo Territoriale ed Ambientale
+
+*/
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -310,9 +319,14 @@ gaiaDxfWriteRing (gaiaDxfWriterPtr dxf, const char *layer, gaiaRingPtr ring)
     fprintf (dxf->out, "%3d\r\nPOLYLINE\r\n%3d\r\n%s\r\n%3d\r\n%6d\r\n", 0, 8,
 	     layer, 66, 1);
     fprintf (dxf->out, "%3d\r\n%6d\r\n", 70, 1);
-    for (iv = 0; iv < ring->Points; iv++)
+    for (iv = 0; iv < ring->Points - 1; iv++)
       {
-	  /* exporting all vertices */
+	  /* sandro 2013-10-19 
+	     exporting all vertices except the last one 
+	     because accordingly to DXF specifications the
+	     Ring closure is always implicitly assumed, so 
+	     there is no need at all to explicitly export
+	     a last vertex identical to the first one */
 	  if (ring->DimensionModel == GAIA_XY_Z)
 	    {
 		gaiaGetPointXYZ (ring->Coords, iv, &x, &y, &z);
