@@ -2,7 +2,7 @@
 
  gg_advanced.c -- Gaia advanced geometric operations
   
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -1187,6 +1187,12 @@ gaiaIsToxicRing (gaiaRingPtr ring)
 
 GAIAGEO_DECLARE int
 gaiaIsToxic (gaiaGeomCollPtr geom)
+{
+    return gaiaIsToxic_r (NULL, geom);
+}
+
+GAIAGEO_DECLARE int
+gaiaIsToxic_r (const void *cache, gaiaGeomCollPtr geom)
 {
 /* 
 / identifying toxic geometries 
@@ -1213,8 +1219,13 @@ gaiaIsToxic (gaiaGeomCollPtr geom)
 	  /* checking LINESTRINGs */
 	  if (gaiaIsToxicLinestring (line))
 	    {
-		gaiaSetGeosAuxErrorMsg
-		    ("gaiaIsToxic detected a toxic Linestring: < 2 pts");
+		if (cache != NULL)
+		    gaiaSetGeosAuxErrorMsg_r
+			(cache,
+			 "gaiaIsToxic detected a toxic Linestring: < 2 pts");
+		else
+		    gaiaSetGeosAuxErrorMsg
+			("gaiaIsToxic detected a toxic Linestring: < 2 pts");
 		return 1;
 	    }
 	  line = line->Next;
@@ -1226,8 +1237,12 @@ gaiaIsToxic (gaiaGeomCollPtr geom)
 	  ring = polyg->Exterior;
 	  if (gaiaIsToxicRing (ring))
 	    {
-		gaiaSetGeosAuxErrorMsg
-		    ("gaiaIsToxic detected a toxic Ring: < 4 pts");
+		if (cache != NULL)
+		    gaiaSetGeosAuxErrorMsg_r
+			(cache, "gaiaIsToxic detected a toxic Ring: < 4 pts");
+		else
+		    gaiaSetGeosAuxErrorMsg
+			("gaiaIsToxic detected a toxic Ring: < 4 pts");
 		return 1;
 	    }
 	  for (ib = 0; ib < polyg->NumInteriors; ib++)
@@ -1235,8 +1250,13 @@ gaiaIsToxic (gaiaGeomCollPtr geom)
 		ring = polyg->Interiors + ib;
 		if (gaiaIsToxicRing (ring))
 		  {
-		      gaiaSetGeosAuxErrorMsg
-			  ("gaiaIsToxic detected a toxic Ring: < 4 pts");
+		      if (cache != NULL)
+			  gaiaSetGeosAuxErrorMsg_r
+			      (cache,
+			       "gaiaIsToxic detected a toxic Ring: < 4 pts");
+		      else
+			  gaiaSetGeosAuxErrorMsg
+			      ("gaiaIsToxic detected a toxic Ring: < 4 pts");
 		      return 1;
 		  }
 	    }
@@ -1247,6 +1267,12 @@ gaiaIsToxic (gaiaGeomCollPtr geom)
 
 GAIAGEO_DECLARE int
 gaiaIsNotClosedRing (gaiaRingPtr ring)
+{
+    return gaiaIsNotClosedRing_r (NULL, ring);
+}
+
+GAIAGEO_DECLARE int
+gaiaIsNotClosedRing_r (const void *cache, gaiaRingPtr ring)
 {
 /* checking a Ring for closure */
     double x0;
@@ -1263,18 +1289,29 @@ gaiaIsNotClosedRing (gaiaRingPtr ring)
 	return 0;
     else
       {
-	  gaiaSetGeosAuxErrorMsg ("gaia detected a not-closed Ring");
+	  if (cache != NULL)
+	      gaiaSetGeosAuxErrorMsg_r (cache,
+					"gaia detected a not-closed Ring");
+	  else
+	      gaiaSetGeosAuxErrorMsg ("gaia detected a not-closed Ring");
 	  return 1;
       }
 }
 
 GAIAGEO_DECLARE int
 gaiaIsNotClosedGeomColl (gaiaGeomCollPtr geom)
+{
+    return gaiaIsNotClosedGeomColl_r (NULL, geom);
+}
+
+GAIAGEO_DECLARE int
+gaiaIsNotClosedGeomColl_r (const void *cache, gaiaGeomCollPtr geom)
 {
 /* 
 / identifying not properly closed Rings 
 / i.e. geoms making GEOS to crash !!!
 */
+    int ret;
     int ib;
     gaiaPolygonPtr polyg;
     gaiaRingPtr ring;
@@ -1285,12 +1322,20 @@ gaiaIsNotClosedGeomColl (gaiaGeomCollPtr geom)
       {
 	  /* checking POLYGONs */
 	  ring = polyg->Exterior;
-	  if (gaiaIsNotClosedRing (ring))
+	  if (cache != NULL)
+	      ret = gaiaIsNotClosedRing_r (cache, ring);
+	  else
+	      ret = gaiaIsNotClosedRing (ring);
+	  if (ret)
 	      return 1;
 	  for (ib = 0; ib < polyg->NumInteriors; ib++)
 	    {
 		ring = polyg->Interiors + ib;
-		if (gaiaIsNotClosedRing (ring))
+		if (cache != NULL)
+		    ret = gaiaIsNotClosedRing_r (cache, ring);
+		else
+		    ret = gaiaIsNotClosedRing (ring);
+		if (ret)
 		    return 1;
 	    }
 	  polyg = polyg->Next;
@@ -1426,14 +1471,14 @@ gaiaDissolveSegments (gaiaGeomCollPtr geom)
     gaiaRingPtr rng;
     int iv;
     int ib;
-    double x;
-    double y;
-    double z;
-    double m;
-    double x0;
-    double y0;
-    double z0;
-    double m0;
+    double x = 0.0;
+    double y = 0.0;
+    double z = 0.0;
+    double m = 0.0;
+    double x0 = 0.0;
+    double y0 = 0.0;
+    double z0 = 0.0;
+    double m0 = 0.0;
     if (!geom)
 	return NULL;
 
@@ -1707,10 +1752,10 @@ gaiaDissolvePoints (gaiaGeomCollPtr geom)
     gaiaRingPtr rng;
     int iv;
     int ib;
-    double x;
-    double y;
-    double z;
-    double m;
+    double x = 0.0;
+    double y = 0.0;
+    double z = 0.0;
+    double m = 0.0;
     if (!geom)
 	return NULL;
 
@@ -1878,7 +1923,10 @@ gaiaExtractPointsFromGeomColl (gaiaGeomCollPtr geom)
 	  pt = pt->Next;
       }
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_MULTIPOINT;
+    if (pts == 1)
+	result->DeclaredType = GAIA_POINT;
+    else
+	result->DeclaredType = GAIA_MULTIPOINT;
     return result;
 }
 
@@ -1945,7 +1993,10 @@ gaiaExtractLinestringsFromGeomColl (gaiaGeomCollPtr geom)
 	  ln = ln->Next;
       }
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_MULTILINESTRING;
+    if (lns == 1)
+	result->DeclaredType = GAIA_LINESTRING;
+    else
+	result->DeclaredType = GAIA_MULTILINESTRING;
     return result;
 }
 
@@ -2046,6 +2097,9 @@ gaiaExtractPolygonsFromGeomColl (gaiaGeomCollPtr geom)
 	  pg = pg->Next;
       }
     result->Srid = geom->Srid;
-    result->DeclaredType = GAIA_MULTIPOLYGON;
+    if (pgs == 1)
+	result->DeclaredType = GAIA_POLYGON;
+    else
+	result->DeclaredType = GAIA_MULTIPOLYGON;
     return result;
 }
