@@ -2,7 +2,7 @@
 
  gg_transform.c -- Gaia PROJ.4 wrapping
   
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -46,6 +46,7 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 
 #if defined(_WIN32) && !defined(__MINGW32__)
 #include "config-msvc.h"
@@ -58,6 +59,7 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #endif
 
 #include <spatialite/sqlite.h>
+#include <spatialite_private.h>
 
 #include <spatialite/gaiageo.h>
 
@@ -196,6 +198,12 @@ gaiaMakeEllipticArc (double cx,
 	  y = cy + (y_axis * sin (rads));
 	  gaiaAppendPointToDynamicLine (dyn, x, y);
 	  angle += step;
+	  points++;
+      }
+    if (points == 0)
+      {
+	  gaiaFreeDynamicLine (dyn);
+	  return NULL;
       }
 /* closing the arc */
     rads = stop * .0174532925199432958;
@@ -205,6 +213,7 @@ gaiaMakeEllipticArc (double cx,
 	gaiaAppendPointToDynamicLine (dyn, x, y);
 
     pt = dyn->First;
+    points = 0;
     while (pt)
       {
 	  /* counting how many points */
@@ -770,10 +779,10 @@ gaiaNormalizeLonLat (gaiaGeomCollPtr geom)
 */
     int ib;
     int iv;
-    double x;
-    double y;
-    double z;
-    double m;
+    double x = 0.0;
+    double y = 0.0;
+    double z = 0.0;
+    double m = 0.0;
     gaiaPointPtr point;
     gaiaPolygonPtr polyg;
     gaiaLinestringPtr line;
@@ -1589,8 +1598,9 @@ gaiaDegsToRads (double degs)
     return degs * DEG_TO_RAD;
 }
 
-GAIAGEO_DECLARE gaiaGeomCollPtr
-gaiaTransform (gaiaGeomCollPtr org, char *proj_from, char *proj_to)
+static gaiaGeomCollPtr
+gaiaTransformCommon (projCtx handle, gaiaGeomCollPtr org, char *proj_from,
+		     char *proj_to)
 {
 /* creates a new GEOMETRY reprojecting coordinates from the original one */
     int ib;
@@ -1614,9 +1624,19 @@ gaiaTransform (gaiaGeomCollPtr org, char *proj_from, char *proj_to)
     gaiaPolygonPtr dst_pg;
     gaiaRingPtr rng;
     gaiaRingPtr dst_rng;
-    projPJ from_cs = pj_init_plus (proj_from);
-    projPJ to_cs = pj_init_plus (proj_to);
+    projPJ from_cs;
+    projPJ to_cs;
     gaiaGeomCollPtr dst;
+    if (handle != NULL)
+      {
+	  from_cs = pj_init_plus_ctx (handle, proj_from);
+	  to_cs = pj_init_plus_ctx (handle, proj_to);
+      }
+    else
+      {
+	  from_cs = pj_init_plus (proj_from);
+	  to_cs = pj_init_plus (proj_to);
+      }
     if (!from_cs)
       {
 	  if (to_cs)
@@ -2101,4 +2121,28 @@ gaiaTransform (gaiaGeomCollPtr org, char *proj_from, char *proj_to)
     return dst;
 }
 
+GAIAGEO_DECLARE gaiaGeomCollPtr
+gaiaTransform (gaiaGeomCollPtr org, char *proj_from, char *proj_to)
+{
+    return gaiaTransformCommon (NULL, org, proj_from, proj_to);
+}
+
+GAIAGEO_DECLARE gaiaGeomCollPtr
+gaiaTransform_r (const void *p_cache, gaiaGeomCollPtr org, char *proj_from,
+		 char *proj_to)
+{
+    struct splite_internal_cache *cache =
+	(struct splite_internal_cache *) p_cache;
+    projCtx handle = NULL;
+    if (cache == NULL)
+	return NULL;
+    if (cache->magic1 != SPATIALITE_CACHE_MAGIC1
+	|| cache->magic2 != SPATIALITE_CACHE_MAGIC2)
+	return NULL;
+    handle = cache->PROJ_handle;
+    if (handle == NULL)
+	return NULL;
+    return gaiaTransformCommon (handle, org, proj_from, proj_to);
+}
+
 #endif /* end including PROJ.4 */
