@@ -2,7 +2,7 @@
 
  gg_vanuatu.c -- WKT parser/lexer 
   
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 The Vanuatu Team - University of Toronto - Supervisor:
@@ -214,8 +214,8 @@ vanuatuMapDynAlloc (struct vanuatu_data *p_data, int type, void *ptr)
 	  p_data->vanuatu_last_dyn_block->next = p;
 	  p_data->vanuatu_last_dyn_block = p;
       }
-    p_data->vanuatu_last_dyn_block->type[p_data->vanuatu_last_dyn_block->
-					 index] = type;
+    p_data->vanuatu_last_dyn_block->type[p_data->
+					 vanuatu_last_dyn_block->index] = type;
     p_data->vanuatu_last_dyn_block->ptr[p_data->vanuatu_last_dyn_block->index] =
 	ptr;
     p_data->vanuatu_last_dyn_block->index++;
