@@ -2,7 +2,7 @@
 
  validator.c -- implements geometry validation and repair
 
- version 4.1, 2013 March 19
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Alternatively, the contents of this file may be used under the terms of
@@ -1667,12 +1667,16 @@ update_repaired (sqlite3 * sqlite, const char *table, const char *geometry,
     return;
 }
 
-SPATIALITE_DECLARE int
-sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
-			  const char *tmp_table, const char *report_path,
-			  int *n_invalids, int *n_repaired, int *n_discarded,
-			  int *n_failures, char **err_msg)
+static int
+sanitize_geometry_column_common (const void *p_cache, sqlite3 * sqlite,
+				 const char *table, const char *geom,
+				 const char *tmp_table, const char *report_path,
+				 int *n_invalids, int *n_repaired,
+				 int *n_discarded, int *n_failures,
+				 char **err_msg)
 {
+#ifdef ENABLE_LWGEOM		/* only if LWGEOM is enabled */
+
 /* attempts to repair invalid Geometries from a Geometry Column */
     char *sql;
     char *xtable;
@@ -1704,7 +1708,7 @@ sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
 
     if (!check_table_column (sqlite, table, geom, &gtype, &srid))
       {
-	  spatialite_e ("sanitize_geometry_column error: <%s>\n"
+	  spatialite_e ("sanitize_geometry_column error: <%s><%s>\n"
 			"Not defined in \"geometry_columns\"", table, geom);
 	  if (err_msg != NULL)
 	    {
@@ -1977,8 +1981,18 @@ sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
 		if (geom)
 		  {
 		      /* checking a geometry for validity */
-		      gaiaResetGeosMsg ();
-		      if (!gaiaIsValid (geom))
+		      int valret;
+		      if (p_cache != NULL)
+			{
+			    gaiaResetGeosMsg_r (p_cache);
+			    valret = gaiaIsValid_r (p_cache, geom);
+			}
+		      else
+			{
+			    gaiaResetGeosMsg ();
+			    valret = gaiaIsValid (geom);
+			}
+		      if (!valret)
 			{
 			    unsigned char *blob_geom;
 			    int blob_sz_geom;
@@ -2288,7 +2302,7 @@ sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
     else if (report->n_repaired > 0)
       {
 	  fprintf (out,
-		   "\t\t\t<tr><td colspan=\"2\" class=\"ok\">This layer has been succesfully repaired and is now completely valid;<br>");
+		   "\t\t\t<tr><td colspan=\"2\" class=\"ok\">This layer has been successfully repaired and is now completely valid;<br>");
 	  fprintf (out,
 		   "doesn't require any further corrective action.</td></tr>\n");
       }
@@ -2395,14 +2409,42 @@ sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
     free_sanitize_report (report);
     if (out)
 	fclose (out);
+	
+#endif /* end LWGEOM conditional */
+    
     return 0;
 }
 
 SPATIALITE_DECLARE int
-sanitize_all_geometry_columns (sqlite3 * sqlite,
-			       const char *tmp_prefix,
-			       const char *output_dir, int *x_not_repaired,
-			       char **err_msg)
+sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
+			  const char *tmp_table, const char *report_path,
+			  int *n_invalids, int *n_repaired, int *n_discarded,
+			  int *n_failures, char **err_msg)
+{
+    return sanitize_geometry_column_common (NULL, sqlite, table, geom,
+					    tmp_table, report_path, n_invalids,
+					    n_repaired, n_discarded, n_failures,
+					    err_msg);
+}
+
+SPATIALITE_DECLARE int
+sanitize_geometry_column_r (const void *p_cache, sqlite3 * sqlite,
+			    const char *table, const char *geom,
+			    const char *tmp_table, const char *report_path,
+			    int *n_invalids, int *n_repaired, int *n_discarded,
+			    int *n_failures, char **err_msg)
+{
+    return sanitize_geometry_column_common (p_cache, sqlite, table, geom,
+					    tmp_table, report_path, n_invalids,
+					    n_repaired, n_discarded, n_failures,
+					    err_msg);
+}
+
+static int
+sanitize_all_geometry_columns_common (const void *p_cache, sqlite3 * sqlite,
+				      const char *tmp_prefix,
+				      const char *output_dir,
+				      int *x_not_repaired, char **err_msg)
 {
 /* attempts to repair invalid Geometries from all Geometry Columns */
     const char *sql;
@@ -2566,11 +2608,20 @@ sanitize_all_geometry_columns (sqlite3 * sqlite,
 		report = sqlite3_mprintf ("%s/lyr_%04d.html", output_dir, i);
 		tmp_table =
 		    sqlite3_mprintf ("%s%s_%s", tmp_prefix, table, geom);
-		ret =
-		    sanitize_geometry_column (sqlite, table, geom, tmp_table,
-					      report, &n_invalids, &n_repaired,
-					      &n_discarded, &n_failures,
-					      err_msg);
+		if (p_cache != NULL)
+		    ret =
+			sanitize_geometry_column_r (p_cache, sqlite, table,
+						    geom, tmp_table, report,
+						    &n_invalids, &n_repaired,
+						    &n_discarded, &n_failures,
+						    err_msg);
+		else
+		    ret =
+			sanitize_geometry_column (sqlite, table, geom,
+						  tmp_table, report,
+						  &n_invalids, &n_repaired,
+						  &n_discarded, &n_failures,
+						  err_msg);
 		sqlite3_free (report);
 		sqlite3_free (tmp_table);
 		fprintf (out,
@@ -2586,7 +2637,7 @@ sanitize_all_geometry_columns (sqlite3 * sqlite,
 		else if (n_discarded == 0 && n_failures == 0)
 		  {
 		      p_msg =
-			  "NONE: this layer has been succesfully sanitized and is now fully valid";
+			  "NONE: this layer has been successfully sanitized and is now fully valid";
 		      cls_msg = "ok";
 		  }
 		else if (n_discarded == 0 && n_failures > 0)
@@ -2635,6 +2686,28 @@ sanitize_all_geometry_columns (sqlite3 * sqlite,
     return 0;
 }
 
+SPATIALITE_DECLARE int
+sanitize_all_geometry_columns (sqlite3 * sqlite,
+			       const char *tmp_prefix,
+			       const char *output_dir, int *x_not_repaired,
+			       char **err_msg)
+{
+    return sanitize_all_geometry_columns_common (NULL, sqlite, tmp_prefix,
+						 output_dir, x_not_repaired,
+						 err_msg);
+}
+
+SPATIALITE_DECLARE int
+sanitize_all_geometry_columns_r (const void *p_cache, sqlite3 * sqlite,
+				 const char *tmp_prefix,
+				 const char *output_dir, int *x_not_repaired,
+				 char **err_msg)
+{
+    return sanitize_all_geometry_columns_common (p_cache, sqlite, tmp_prefix,
+						 output_dir, x_not_repaired,
+						 err_msg);
+}
+
 #else /* LIBXML2 isn't enabled */
 
 SPATIALITE_DECLARE int
@@ -2661,6 +2734,30 @@ sanitize_all_geometry_columns (sqlite3 * sqlite,
     return 0;
 }
 
+SPATIALITE_DECLARE int
+sanitize_all_geometry_columns_r (const void *p_cache, sqlite3 * sqlite,
+				 const char *tmp_prefix,
+				 const char *output_dir, int *x_not_repaired,
+				 char **err_msg)
+{
+/* LWGEOM isn't enabled: always returning an error */
+    int len;
+    const char *msg = "Sorry ... libspatialite was built disabling LWGEOM\n"
+	"and is thus unable to support MakeValid";
+
+/* silencing stupid compiler warnings */
+    if (p_cache == NULL || sqlite == NULL || tmp_prefix == NULL
+	|| output_dir == NULL || x_not_repaired == NULL)
+	tmp_prefix = NULL;
+
+    if (err_msg == NULL)
+	return 0;
+    len = strlen (msg);
+    *err_msg = malloc (len + 1);
+    strcpy (*err_msg, msg);
+    return 0;
+}
+
 SPATIALITE_DECLARE int
 sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
 			  const char *tmp_table, const char *report_path,
@@ -2687,6 +2784,33 @@ sanitize_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
     return 0;
 }
 
+SPATIALITE_DECLARE int
+sanitize_geometry_column_r (const void *p_cache, sqlite3 * sqlite,
+			    const char *table, const char *geom,
+			    const char *tmp_table, const char *report_path,
+			    int *n_invalids, int *n_repaired, int *n_discarded,
+			    int *n_failures, char **err_msg)
+{
+/* LWGEOM isn't enabled: always returning an error */
+    int len;
+    const char *msg = "Sorry ... libspatialite was built disabling LWGEOM\n"
+	"and is thus unable to support MakeValid";
+
+/* silencing stupid compiler warnings */
+    if (p_cache == NULL || sqlite == NULL || table == NULL || geom == NULL
+	|| tmp_table == NULL || report_path == NULL || n_invalids == NULL
+	|| n_repaired == NULL || n_discarded == NULL || n_failures == NULL)
+	table = NULL;
+
+
+    if (err_msg == NULL)
+	return 0;
+    len = strlen (msg);
+    *err_msg = malloc (len + 1);
+    strcpy (*err_msg, msg);
+    return 0;
+}
+
 #endif /* end LWGEOM conditionals */
 
 
@@ -2773,10 +2897,11 @@ addMessageToValidityReport (struct validity_report *report, sqlite3_int64 rowid,
     report->last = r;
 }
 
-SPATIALITE_DECLARE int
-check_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
-		       const char *report_path, int *n_rows, int *n_invalids,
-		       char **err_msg)
+static int
+check_geometry_column_common (const void *p_cache, sqlite3 * sqlite,
+			      const char *table, const char *geom,
+			      const char *report_path, int *n_rows,
+			      int *n_invalids, char **err_msg)
 {
 /* checks a Geometry Column for validity */
     char *sql;
@@ -3011,11 +3136,22 @@ check_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
 		      const char *error;
 		      const char *warning;
 		      const char *extra;
-		      gaiaResetGeosMsg ();
-		      valid = gaiaIsValid (geom);
-		      error = gaiaGetGeosErrorMsg ();
-		      warning = gaiaGetGeosWarningMsg ();
-		      extra = gaiaGetGeosAuxErrorMsg ();
+		      if (p_cache != NULL)
+			{
+			    gaiaResetGeosMsg_r (p_cache);
+			    valid = gaiaIsValid_r (p_cache, geom);
+			    error = gaiaGetGeosErrorMsg_r (p_cache);
+			    warning = gaiaGetGeosWarningMsg_r (p_cache);
+			    extra = gaiaGetGeosAuxErrorMsg_r (p_cache);
+			}
+		      else
+			{
+			    gaiaResetGeosMsg ();
+			    valid = gaiaIsValid (geom);
+			    error = gaiaGetGeosErrorMsg ();
+			    warning = gaiaGetGeosWarningMsg ();
+			    extra = gaiaGetGeosAuxErrorMsg ();
+			}
 		      if (!valid || error || warning)
 			  addMessageToValidityReport (report, rowid, valid,
 						      error, warning, extra);
@@ -3282,9 +3418,29 @@ check_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
 }
 
 SPATIALITE_DECLARE int
-check_all_geometry_columns (sqlite3 * sqlite,
-			    const char *output_dir, int *x_invalids,
-			    char **err_msg)
+check_geometry_column (sqlite3 * sqlite, const char *table, const char *geom,
+		       const char *report_path, int *n_rows, int *n_invalids,
+		       char **err_msg)
+{
+    return check_geometry_column_common (NULL, sqlite, table, geom, report_path,
+					 n_rows, n_invalids, err_msg);
+}
+
+SPATIALITE_DECLARE int
+check_geometry_column_r (const void *p_cache, sqlite3 * sqlite,
+			 const char *table, const char *geom,
+			 const char *report_path, int *n_rows, int *n_invalids,
+			 char **err_msg)
+{
+    return check_geometry_column_common (p_cache, sqlite, table, geom,
+					 report_path, n_rows, n_invalids,
+					 err_msg);
+}
+
+static int
+check_all_geometry_columns_common (const void *p_cache, sqlite3 * sqlite,
+				   const char *output_dir, int *x_invalids,
+				   char **err_msg)
 {
 /* checks all Geometry Columns for validity */
     const char *sql;
@@ -3432,9 +3588,15 @@ check_all_geometry_columns (sqlite3 * sqlite,
 		const char *table = results[(i * columns) + 0];
 		const char *geom = results[(i * columns) + 1];
 		report = sqlite3_mprintf ("%s/lyr_%04d.html", output_dir, i);
-		ret =
-		    check_geometry_column (sqlite, table, geom, report, &n_rows,
-					   &n_invalids, err_msg);
+		if (p_cache != NULL)
+		    ret =
+			check_geometry_column_r (p_cache, sqlite, table, geom,
+						 report, &n_rows, &n_invalids,
+						 err_msg);
+		else
+		    ret =
+			check_geometry_column (sqlite, table, geom, report,
+					       &n_rows, &n_invalids, err_msg);
 		sqlite3_free (report);
 		fprintf (out,
 			 "\t\t\t<tr><td align=\"center\"><a href=\"./lyr_%04d.html\">show</a></td>",
@@ -3472,6 +3634,23 @@ check_all_geometry_columns (sqlite3 * sqlite,
     return 0;
 }
 
+SPATIALITE_DECLARE int
+check_all_geometry_columns (sqlite3 * sqlite,
+			    const char *output_dir, int *x_invalids,
+			    char **err_msg)
+{
+    return check_all_geometry_columns_common (NULL, sqlite, output_dir,
+					      x_invalids, err_msg);
+}
+
+SPATIALITE_DECLARE int
+check_all_geometry_columns_r (const void *p_cache, sqlite3 * sqlite,
+			      const char *output_dir, int *x_invalids,
+			      char **err_msg)
+{
+    return check_all_geometry_columns_common (p_cache, sqlite, output_dir,
+					      x_invalids, err_msg);
+}
 
 #else
 
@@ -3496,6 +3675,28 @@ check_all_geometry_columns (sqlite3 * sqlite,
     return 0;
 }
 
+SPATIALITE_DECLARE int
+check_all_geometry_columns_r (const void *p_cache, sqlite3 * sqlite,
+			      const char *output_dir, int *x_invalids,
+			      char **err_msg)
+{
+/* GEOS isn't enabled: always returning an error */
+    int len;
+    const char *msg = "Sorry ... libspatialite was built disabling LWGEOM\n"
+	"and is thus unable to support IsValid";
+/* silencing stupid compiler warnings */
+    if (p_cache == NULL || sqlite == NULL || output_dir == NULL
+	|| x_invalids == NULL)
+	output_dir = NULL;
+
+    if (err_msg == NULL)
+	return 0;
+    len = strlen (msg);
+    *err_msg = malloc (len + 1);
+    strcpy (*err_msg, msg);
+    return 0;
+}
+
 SPATIALITE_DECLARE int
 check_geometry_column (sqlite3 * sqlite,
 		       const char *table,
@@ -3510,7 +3711,32 @@ check_geometry_column (sqlite3 * sqlite,
 
 /* silencing stupid compiler warnings */
     if (sqlite == NULL || table == NULL || geom == NULL ||
-	||report_path == NULL || n_rows == NULL || n_invalids == NULL)
+	report_path == NULL || n_rows == NULL || n_invalids == NULL)
+	table = NULL;
+
+    if (err_msg == NULL)
+	return 0;
+    len = strlen (msg);
+    *err_msg = malloc (len + 1);
+    strcpy (*err_msg, msg);
+    return 0;
+}
+
+SPATIALITE_DECLARE int
+check_geometry_column_r (const void *p_cache, sqlite3 * sqlite,
+			 const char *table,
+			 const char *geom,
+			 const char *report_path,
+			 int *n_rows, int *n_invalids, char **err_msg)
+{
+/* GEOS isn't enabled: always returning an error */
+    int len;
+    const char *msg = "Sorry ... libspatialite was built disabling GEOS\n"
+	"and is thus unable to support IsValid";
+
+/* silencing stupid compiler warnings */
+    if (p_cache == NULL || sqlite == NULL || table == NULL || geom == NULL ||
+	report_path == NULL || n_rows == NULL || n_invalids == NULL)
 	table = NULL;
 
     if (err_msg == NULL)
