@@ -2,7 +2,7 @@
 
  virtualXLc -- SQLite3 extension [VIRTUAL TABLE accessing .XLS]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -900,8 +900,19 @@ vXL_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
-sqlite3VirtualXLInit (sqlite3 * db)
+static int
+vXL_rename (sqlite3_vtab * pVTab, const char *zNew)
+{
+/* BEGIN TRANSACTION */
+    if (pVTab)
+	pVTab = pVTab;		/* unused arg warning suppression */
+    if (zNew)
+	zNew = zNew;		/* unused arg warning suppression */
+    return SQLITE_ERROR;
+}
+
+static int
+spliteVirtualXLInit (sqlite3 * db)
 {
     int rc = SQLITE_OK;
     my_XL_module.iVersion = 1;
@@ -923,14 +934,16 @@ sqlite3VirtualXLInit (sqlite3 * db)
     my_XL_module.xCommit = &vXL_commit;
     my_XL_module.xRollback = &vXL_rollback;
     my_XL_module.xFindFunction = NULL;
+    my_XL_module.xRename = &vXL_rename;
     sqlite3_create_module_v2 (db, "VirtualXL", &my_XL_module, NULL, 0);
     return rc;
 }
 
-int
-virtualXL_extension_init (sqlite3 * db)
+SPATIALITE_PRIVATE int
+virtualXL_extension_init (void *xdb)
 {
-    return sqlite3VirtualXLInit (db);
+    sqlite3 *db = (sqlite3 *) xdb;
+    return spliteVirtualXLInit (db);
 }
 
 #endif /* FreeXL enabled/disabled */
