@@ -2,7 +2,7 @@
 
  virtualdbf.c -- SQLite3 extension [VIRTUAL TABLE accessing DBF]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -77,6 +77,7 @@ typedef struct VirtualDbfStruct
     char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
     sqlite3 *db;		/* the sqlite db holding the virtual table */
     gaiaDbfPtr dbf;		/* the DBF struct */
+    int text_dates;
 } VirtualDbf;
 typedef VirtualDbf *VirtualDbfPtr;
 
@@ -123,13 +124,14 @@ vdbf_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     int seed;
     int dup;
     int idup;
+    int text_dates = 0;
     char *xname;
     char **col_name = NULL;
     gaiaOutBuffer sql_statement;
     if (pAux)
 	pAux = pAux;		/* unused arg warning suppression */
 /* checking for DBF PATH */
-    if (argc == 5)
+    if (argc == 5 || argc == 6)
       {
 	  pPath = argv[3];
 	  len = strlen (pPath);
@@ -156,6 +158,8 @@ vdbf_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	    }
 	  else
 	      strcpy (encoding, pEncoding);
+	  if (argc == 6)
+	      text_dates = atoi (argv[5]);
       }
     else
       {
@@ -172,6 +176,7 @@ vdbf_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     p_vt->zErrMsg = NULL;
     p_vt->db = db;
     p_vt->dbf = gaiaAllocDbf ();
+    p_vt->text_dates = text_dates;
 /* trying to open file */
     gaiaOpenDbfRead (p_vt->dbf, path, encoding, "UTF-8");
     if (!(p_vt->dbf->Valid))
@@ -239,6 +244,15 @@ vdbf_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	    }
 	  else if (pFld->Type == 'F')
 	      sql = sqlite3_mprintf (", \"%s\" DOUBLE", xname);
+	  else if (pFld->Type == 'D')
+	    {
+		if (text_dates)
+		    sql =
+			sqlite3_mprintf (", \"%s\" VARCHAR(%d)", xname,
+					 pFld->Length);
+		else
+		    sql = sqlite3_mprintf (", \"%s\" DOUBLE", xname);
+	    }
 	  else
 	      sql =
 		  sqlite3_mprintf (", \"%s\" VARCHAR(%d)", xname, pFld->Length);
@@ -344,7 +358,9 @@ vdbf_read_row (VirtualDbfCursorPtr cursor, int *deleted_row)
 	  cursor->eof = 1;
 	  return;
       }
-    ret = gaiaReadDbfEntity (cursor->pVtab->dbf, cursor->current_row, &deleted);
+    ret =
+	gaiaReadDbfEntity_ex (cursor->pVtab->dbf, cursor->current_row, &deleted,
+			      cursor->pVtab->text_dates);
     if (!ret)
       {
 	  if (!(cursor->pVtab->dbf->LastError))	/* normal DBF EOF */
@@ -870,8 +886,19 @@ vdbf_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
-sqlite3VirtualDbfInit (sqlite3 * db)
+static int
+vdbf_rename (sqlite3_vtab * pVTab, const char *zNew)
+{
+/* BEGIN TRANSACTION */
+    if (pVTab)
+	pVTab = pVTab;		/* unused arg warning suppression */
+    if (zNew)
+	zNew = zNew;		/* unused arg warning suppression */
+    return SQLITE_ERROR;
+}
+
+static int
+spliteVirtualDbfInit (sqlite3 * db)
 {
     int rc = SQLITE_OK;
     my_dbf_module.iVersion = 1;
@@ -893,14 +920,16 @@ sqlite3VirtualDbfInit (sqlite3 * db)
     my_dbf_module.xCommit = &vdbf_commit;
     my_dbf_module.xRollback = &vdbf_rollback;
     my_dbf_module.xFindFunction = NULL;
+    my_dbf_module.xRename = &vdbf_rename;
     sqlite3_create_module_v2 (db, "VirtualDbf", &my_dbf_module, NULL, 0);
     return rc;
 }
 
-int
-virtualdbf_extension_init (sqlite3 * db)
+SPATIALITE_PRIVATE int
+virtualdbf_extension_init (void *xdb)
 {
-    return sqlite3VirtualDbfInit (db);
+    sqlite3 *db = (sqlite3 *) xdb;
+    return spliteVirtualDbfInit (db);
 }
 
 #endif /* ICONV enabled/disabled */
