@@ -2,7 +2,7 @@
 
  virtualfdo.c -- SQLite3 extension [VIRTUAL TABLE accessing FDO-OGR tables]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -955,8 +955,7 @@ vfdo_insert_row (VirtualFDOPtr p_vt, sqlite3_int64 * rowid, int argc,
 					  {
 					      sqlite3_bind_text (stmt, i - 1,
 								 out_buf.Buffer,
-								 out_buf.
-								 WriteOffset,
+								 out_buf.WriteOffset,
 								 free);
 					      out_buf.Buffer = NULL;
 					      gaiaOutBufferReset (&out_buf);
@@ -1116,7 +1115,7 @@ vfdo_update_row (VirtualFDOPtr p_vt, sqlite3_int64 rowid, int argc,
     char *sql;
     char buf[256];
     char *xname;
-    gaiaGeomCollPtr geom;
+    gaiaGeomCollPtr geom = NULL;
     gaiaOutBufferInitialize (&sql_statement);
     xname = gaiaDoubleQuotedSql (p_vt->table);
     sql = sqlite3_mprintf ("UPDATE \"%s\" SET", xname);
@@ -1190,8 +1189,7 @@ vfdo_update_row (VirtualFDOPtr p_vt, sqlite3_int64 rowid, int argc,
 					  {
 					      sqlite3_bind_text (stmt, i - 1,
 								 out_buf.Buffer,
-								 out_buf.
-								 WriteOffset,
+								 out_buf.WriteOffset,
 								 free);
 					      out_buf.Buffer = NULL;
 					      gaiaOutBufferReset (&out_buf);
@@ -1505,8 +1503,10 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					      if (wkt == NULL)
 						{
 						    value_set_null (*
-								    (cursor->pVtab->Value
-								     + ic));
+								    (cursor->
+								     pVtab->
+								     Value +
+								     ic));
 						    continue;
 						}
 					      delete_wkt = 1;
@@ -1516,9 +1516,8 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					    free ((void *) wkt);
 					if (!geom)
 					    value_set_null (*
-							    (cursor->
-							     pVtab->Value +
-							     ic));
+							    (cursor->pVtab->
+							     Value + ic));
 					else
 					  {
 					      geom->Srid =
@@ -1529,16 +1528,18 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					      if (xblob)
 						{
 						    value_set_blob (*
-								    (cursor->pVtab->Value
-								     + ic),
-								    xblob,
+								    (cursor->
+								     pVtab->
+								     Value +
+								     ic), xblob,
 								    size);
 						    free (xblob);
 						}
 					      else
 						  value_set_null (*
-								  (cursor->pVtab->Value
-								   + ic));
+								  (cursor->
+								   pVtab->
+								   Value + ic));
 					      gaiaFreeGeomColl (geom);
 					  }
 				    }
@@ -1559,9 +1560,8 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					geom = gaiaFromWkb (blob, size);
 					if (!geom)
 					    value_set_null (*
-							    (cursor->
-							     pVtab->Value +
-							     ic));
+							    (cursor->pVtab->
+							     Value + ic));
 					else
 					  {
 					      geom->Srid =
@@ -1572,16 +1572,18 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					      if (xblob)
 						{
 						    value_set_blob (*
-								    (cursor->pVtab->Value
-								     + ic),
-								    xblob,
+								    (cursor->
+								     pVtab->
+								     Value +
+								     ic), xblob,
 								    size);
 						    free (xblob);
 						}
 					      else
 						  value_set_null (*
-								  (cursor->pVtab->Value
-								   + ic));
+								  (cursor->
+								   pVtab->
+								   Value + ic));
 					      gaiaFreeGeomColl (geom);
 					  }
 				    }
@@ -1602,9 +1604,8 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					geom = gaiaFromFgf (blob, size);
 					if (!geom)
 					    value_set_null (*
-							    (cursor->
-							     pVtab->Value +
-							     ic));
+							    (cursor->pVtab->
+							     Value + ic));
 					else
 					  {
 					      geom->Srid =
@@ -1615,16 +1616,18 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
 					      if (xblob)
 						{
 						    value_set_blob (*
-								    (cursor->pVtab->Value
-								     + ic),
-								    xblob,
+								    (cursor->
+								     pVtab->
+								     Value +
+								     ic), xblob,
 								    size);
 						    free (xblob);
 						}
 					      else
 						  value_set_null (*
-								  (cursor->pVtab->Value
-								   + ic));
+								  (cursor->
+								   pVtab->
+								   Value + ic));
 					      gaiaFreeGeomColl (geom);
 					  }
 				    }
@@ -1697,7 +1700,6 @@ vfdo_read_row (VirtualFDOCursorPtr cursor)
     cursor->current_row = pk;
 }
 
-
 static int
 vfdo_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	     sqlite3_vtab ** ppVTab, char **pzErr)
@@ -1725,6 +1727,7 @@ vfdo_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     VirtualFDOPtr p_vt = NULL;
     if (pAux)
 	pAux = pAux;		/* unused arg warning suppression */
+    gaiaOutBufferInitialize (&sql_statement);
 /* checking for table_name */
     if (argc == 4)
       {
@@ -1852,7 +1855,6 @@ vfdo_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
     else
 	goto illegal;
 /* preparing the COLUMNs for this VIRTUAL TABLE */
-    gaiaOutBufferInitialize (&sql_statement);
     xname = gaiaDoubleQuotedSql (vtable);
     sql = sqlite3_mprintf ("CREATE TABLE \"%s\" ", xname);
     free (xname);
@@ -2159,8 +2161,19 @@ vfdo_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
-sqlite3VirtualFDOInit (sqlite3 * db)
+static int
+vfdo_rename (sqlite3_vtab * pVTab, const char *zNew)
+{
+/* BEGIN TRANSACTION */
+    if (pVTab)
+	pVTab = pVTab;		/* unused arg warning suppression */
+    if (zNew)
+	zNew = zNew;		/* unused arg warning suppression */
+    return SQLITE_ERROR;
+}
+
+static int
+spliteVirtualFDOInit (sqlite3 * db)
 {
     int rc = SQLITE_OK;
     my_fdo_module.iVersion = 1;
@@ -2182,12 +2195,14 @@ sqlite3VirtualFDOInit (sqlite3 * db)
     my_fdo_module.xCommit = &vfdo_commit;
     my_fdo_module.xRollback = &vfdo_rollback;
     my_fdo_module.xFindFunction = NULL;
+    my_fdo_module.xRename = &vfdo_rename;
     sqlite3_create_module_v2 (db, "VirtualFDO", &my_fdo_module, NULL, 0);
     return rc;
 }
 
-int
-virtualfdo_extension_init (sqlite3 * db)
+SPATIALITE_PRIVATE int
+virtualfdo_extension_init (void *xdb)
 {
-    return sqlite3VirtualFDOInit (db);
+    sqlite3 *db = (sqlite3 *) xdb;
+    return spliteVirtualFDOInit (db);
 }
