@@ -2,7 +2,7 @@
 
  virtualspatialindex.c -- SQLite3 extension [VIRTUAL TABLE RTree metahandler]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -173,6 +173,12 @@ vspidx_check_view_rtree (sqlite3 * sqlite, const char *table_name,
     sqlite3_finalize (stmt);
     if (count != 1)
 	return 0;
+    if (!validateRowid (sqlite, table_name))
+      {
+	  free (rt);
+	  free (rg);
+	  return 0;
+      }
     *real_table = rt;
     *real_geom = rg;
     return 1;
@@ -888,8 +894,19 @@ vspidx_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
-sqlite3VirtualSpatialIndexInit (sqlite3 * db)
+static int
+vspidx_rename (sqlite3_vtab * pVTab, const char *zNew)
+{
+/* BEGIN TRANSACTION */
+    if (pVTab)
+	pVTab = pVTab;		/* unused arg warning suppression */
+    if (zNew)
+	zNew = zNew;		/* unused arg warning suppression */
+    return SQLITE_ERROR;
+}
+
+static int
+spliteVirtualSpatialIndexInit (sqlite3 * db)
 {
     int rc = SQLITE_OK;
     my_spidx_module.iVersion = 1;
@@ -911,13 +928,15 @@ sqlite3VirtualSpatialIndexInit (sqlite3 * db)
     my_spidx_module.xCommit = &vspidx_commit;
     my_spidx_module.xRollback = &vspidx_rollback;
     my_spidx_module.xFindFunction = NULL;
+    my_spidx_module.xRename = &vspidx_rename;
     sqlite3_create_module_v2 (db, "VirtualSpatialIndex", &my_spidx_module, NULL,
 			      0);
     return rc;
 }
 
-int
-virtual_spatialindex_extension_init (sqlite3 * db)
+SPATIALITE_PRIVATE int
+virtual_spatialindex_extension_init (void *xdb)
 {
-    return sqlite3VirtualSpatialIndexInit (db);
+    sqlite3 *db = (sqlite3 *) xdb;
+    return spliteVirtualSpatialIndexInit (db);
 }
