@@ -2,7 +2,7 @@
 
  virtualxpath.c -- SQLite3 extension [VIRTUAL XPath handler]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -72,9 +72,9 @@ the terms of any one of the MPL, the GPL or the LGPL.
 #include <spatialite/sqlite.h>
 
 #include <spatialite/spatialite.h>
+#include <spatialite/debug.h>
 #include <spatialite/gaiaaux.h>
 #include <spatialite/gaiageo.h>
-#include <spatialite_private.h>
 
 #ifdef _WIN32
 #define strcasecmp	_stricmp
@@ -96,7 +96,7 @@ typedef struct VirtualXPathStruct
     int nRef;			/* # references: USED INTERNALLY BY SQLITE */
     char *zErrMsg;		/* error message: USE INTERNALLY BY SQLITE */
     sqlite3 *db;		/* the sqlite db holding the virtual table */
-    void *p_cache;		/* pointer to the internal cache */
+    const void *p_cache;	/* pointer to the internal cache */
     char *table;		/* the real-table name */
     char *column;		/* the real-column name */
 } VirtualXPath;
@@ -230,10 +230,10 @@ vxpath_feed_ns (struct vxpath_namespaces *ns_list, xmlNodePtr start)
 			      {
 				  /* a Namespace is defined */
 				  vxpath_add_ns (ns_list,
-						 (const char *) (attr->
-								 ns->prefix),
-						 (const char *) (attr->
-								 ns->href));
+						 (const char *) (attr->ns->
+								 prefix),
+						 (const char *) (attr->ns->
+								 href));
 			      }
 			}
 		      attr = attr->next;
@@ -258,17 +258,32 @@ vxpath_get_namespaces (void *p_xml_doc)
     return ns_list;
 }
 
+static int
+is_valid_cache (struct splite_internal_cache *cache)
+{
+/* testing if the passed cache is a valid one */
+    if (cache == NULL)
+	return 0;
+    if (cache->magic1 != SPATIALITE_CACHE_MAGIC1
+	|| cache->magic2 != SPATIALITE_CACHE_MAGIC2)
+	return 0;
+    return 1;
+}
+
 static void
 vxpathError (void *ctx, const char *msg, ...)
 {
 /* appending to the current XPath Error buffer */
     struct splite_internal_cache *cache = (struct splite_internal_cache *) ctx;
-    gaiaOutBufferPtr buf = (gaiaOutBufferPtr) (cache->xmlXPathErrors);
+    gaiaOutBufferPtr buf;
     char out[65536];
     va_list args;
 
     if (ctx != NULL)
 	ctx = NULL;		/* suppressing stupid compiler warnings (unused args) */
+    if (!is_valid_cache (cache))
+	return;
+    buf = (gaiaOutBufferPtr) (cache->xmlXPathErrors);
 
     va_start (args, msg);
     vsnprintf (out, 65536, msg, args);
@@ -280,17 +295,23 @@ static void
 vxpathResetXmlErrors (struct splite_internal_cache *cache)
 {
 /* resetting the XPath Error buffer */
-    gaiaOutBufferPtr buf = (gaiaOutBufferPtr) (cache->xmlXPathErrors);
+    gaiaOutBufferPtr buf;
+    if (!is_valid_cache (cache))
+	return;
+    buf = (gaiaOutBufferPtr) (cache->xmlXPathErrors);
     gaiaOutBufferReset (buf);
 }
 
 GAIAGEO_DECLARE int
-gaiaIsValidXPathExpression (void *p_cache, const char *xpath_expr)
+gaiaIsValidXPathExpression (const void *p_cache, const char *xpath_expr)
 {
     struct splite_internal_cache *cache =
 	(struct splite_internal_cache *) p_cache;
     xmlXPathCompExprPtr result;
-    xmlGenericErrorFunc xpathError = (xmlGenericErrorFunc) vxpathError;
+    xmlGenericErrorFunc xpathError;
+    if (!is_valid_cache (cache))
+	return 0;
+    xpathError = (xmlGenericErrorFunc) vxpathError;
 
     vxpathResetXmlErrors (cache);
     xmlSetGenericErrorFunc (cache, xpathError);
@@ -307,7 +328,7 @@ gaiaIsValidXPathExpression (void *p_cache, const char *xpath_expr)
 }
 
 SPATIALITE_PRIVATE int
-vxpath_eval_expr (void *p_cache, void *x_xml_doc, const char *xpath_expr,
+vxpath_eval_expr (const void *p_cache, void *x_xml_doc, const char *xpath_expr,
 		  void *x_xpathCtx, void *x_xpathObj)
 {
     struct splite_internal_cache *cache =
@@ -324,8 +345,11 @@ vxpath_eval_expr (void *p_cache, void *x_xml_doc, const char *xpath_expr,
     struct vxpath_ns *ns;
     struct vxpath_namespaces *ns_list = vxpath_get_namespaces (xml_doc);
 
-    vxpathResetXmlErrors (cache);
-    xmlSetGenericErrorFunc (cache, xpathError);
+    if (is_valid_cache (cache))
+      {
+	  vxpathResetXmlErrors (cache);
+	  xmlSetGenericErrorFunc (cache, xpathError);
+      }
 
 /* creating an XPath context */
     xpathCtx = xmlXPathNewContext (xml_doc);
@@ -580,6 +604,8 @@ vxpath_create (sqlite3 * db, void *pAux, int argc, const char *const *argv,
 	return SQLITE_NOMEM;
     p_vt->db = db;
     p_vt->p_cache = pAux;
+    if (p_vt->p_cache == NULL)
+	spatialite_e ("VirtualXPath WARNING - no XML cache is available !!!\n");
     p_vt->nRef = 0;
     p_vt->zErrMsg = NULL;
     p_vt->table = table;
@@ -1085,8 +1111,19 @@ vxpath_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
-sqlite3VirtualXPathInit (sqlite3 * db, void *p_cache)
+static int
+vxpath_rename (sqlite3_vtab * pVTab, const char *zNew)
+{
+/* BEGIN TRANSACTION */
+    if (pVTab)
+	pVTab = pVTab;		/* unused arg warning suppression */
+    if (zNew)
+	zNew = zNew;		/* unused arg warning suppression */
+    return SQLITE_ERROR;
+}
+
+static int
+spliteVirtualXPathInit (sqlite3 * db, void *p_cache)
 {
     int rc = SQLITE_OK;
     my_xpath_module.iVersion = 1;
@@ -1108,14 +1145,16 @@ sqlite3VirtualXPathInit (sqlite3 * db, void *p_cache)
     my_xpath_module.xCommit = &vxpath_commit;
     my_xpath_module.xRollback = &vxpath_rollback;
     my_xpath_module.xFindFunction = NULL;
+    my_xpath_module.xRename = &vxpath_rename;
     sqlite3_create_module_v2 (db, "VirtualXPath", &my_xpath_module, p_cache, 0);
     return rc;
 }
 
-int
-virtual_xpath_extension_init (sqlite3 * db, void *p_cache)
+SPATIALITE_PRIVATE int
+virtual_xpath_extension_init (void *xdb, const void *p_cache)
 {
-    return sqlite3VirtualXPathInit (db, p_cache);
+    sqlite3 *db = (sqlite3 *) xdb;
+    return spliteVirtualXPathInit (db, (void *) p_cache);
 }
 
 #endif /* end LIBXML2: supporting XML documents */
