@@ -2,7 +2,7 @@
 
  virtualtext.c -- SQLite3 extension [VIRTUAL TABLE accessing CSV/TXT]
 
- version 4.1, 2013 May 8
+ version 4.3, 2015 June 29
 
  Author: Sandro Furieri a.furieri@lqt.it
 
@@ -24,7 +24,7 @@ The Original Code is the SpatiaLite library
 
 The Initial Developer of the Original Code is Alessandro Furieri
  
-Portions created by the Initial Developer are Copyright (C) 2008-2013
+Portions created by the Initial Developer are Copyright (C) 2008-2015
 the Initial Developer. All Rights Reserved.
 
 Contributor(s):
@@ -942,7 +942,7 @@ vtxt_rollback (sqlite3_vtab * pVTab)
     return SQLITE_OK;
 }
 
-int
+static int
 sqlite3VirtualTextInit (sqlite3 * db)
 {
     int rc = SQLITE_OK;
@@ -969,9 +969,10 @@ sqlite3VirtualTextInit (sqlite3 * db)
     return rc;
 }
 
-int
-virtualtext_extension_init (sqlite3 * db)
+SPATIALITE_PRIVATE int
+virtualtext_extension_init (void *xdb)
 {
+    sqlite3 *db = (sqlite3 *) xdb;
     return sqlite3VirtualTextInit (db);
 }
 
@@ -1149,7 +1150,7 @@ vrttxt_is_integer (const char *value)
 	  p++;
       }
     if (invalids)
-	return 0;		/* invalid chars where found */
+	return 0;		/* invalid chars were found */
     if (signs > 1)
 	return 0;		/* more than a single sign */
     if (signs)
@@ -1199,7 +1200,7 @@ vrttxt_is_double (const char *value, char decimal_separator)
 	  p++;
       }
     if (invalids)
-	return 0;		/* invalid chars where found */
+	return 0;		/* invalid chars were found */
     if (points > 1)
 	return 0;		/* more than a single decimal separator */
     if (signs > 1)
@@ -1236,6 +1237,8 @@ vrttxt_set_column_title (gaiaTextReaderPtr txt, int col_no, const char *name)
     char *utf8text;
     char *str = (char *) name;
     int len = strlen (str);
+    if (len <= 0)
+	return 0;
     if (str[0] == txt->text_separator && str[len - 1] == txt->text_separator)
       {
 	  /* cleaning the enclosing quotes */
